MOBILE_USERAGENT = 'Reddit/Version 2020.47.0/Build 302202/Android 11'
WEB_USERAGENT = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:85.0) Gecko/20100101 Firefox/85.0'
OAUTH_CLIENT_ID_B64 = "b2hYcG9xclpZdWIxa2c6"
USER_AGENT = 'Jand/3.0.82'
SB_User_Agent = 'Android%2Fc3.0.144F'
SB_ai = '2515BDA8-9D3A-47CF-9325-330BC37ADA13'
SB_PROXY_CHATMEDIA = 'https://sendbirdproxyk8s.chat.redditmedia.com'
S_REDDIT = 'https://s.reddit.com'
OAUTH_REDDIT = 'https://oauth.reddit.com'
WWW_REDDIT = 'https://old.reddit.com'
ACCOUNTS_REDDIT = 'https://accounts.reddit.com'

# ws payloads
MESG_regular = 'MESG{{"channel_url":"{channel_url}","message":"{text}","data":"{{\\"v1\\":{{\\"preview_collapsed\\":false,\\"embed_data\\":{{}},\\"hidden\\":false,\\"highlights\\":[],\\"message_body\\":\\"{text}\\"}}}}","mention_type":"users","req_id":"{req_id}"}}\n'
MESG_snoo = 'MESG{{"channel_url":"{channel_url}","message":"","data":"{{\\"v1\\":{{\\"preview_collapsed\\":false,\\"embed_data\\":{{\\"site_name\\":\\"Reddit\\"}},\\"hidden\\":false,\\"snoomoji\\":\\"{snoomoji}\\"}}}}","mention_type":"users","req_id":"{req_id}"}}\n'
TPST = 'TPST{{"channel_url":"{channel_url}","time":{time},"req_id":""}}\n'
TPEN = 'TPEN{{"channel_url":"{channel_url}","time":{time},"req_id":""}}\n'
